@@ -7,35 +7,40 @@ package main
 import (
 	"bytes"
 	"encoding/xml"
+	"errors"
 	"fmt"
-	"log"
 	"net/http"
+	"net/url"
 	"os"
 	"path/filepath"
 	"regexp"
 	"strconv"
 	"strings"
+	"time"
 
 	"github.com/beevik/etree"            // BSD-2-clause
 	"github.com/gabriel-vasile/mimetype" // MIT License
-	"github.com/mattn/go-xmpp"           // BSD-3-Clause
+	"github.com/xmppo/go-xmpp"           // BSD-3-Clause
 )
 
-func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath string) string {
+func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath string,
+	timeout time.Duration,
+) (string, error) {
 	var uploadComponent string
 	var maxFileSize int64
+	var iqDiscoItemsXMLQuery, iqDiscoInfoXMLQuery *etree.Element
 
 	// Get file size
 	fileInfo, err := os.Stat(filePath)
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 	fileSize := fileInfo.Size()
 
 	// Read file
 	buffer, err := readFile(filePath)
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 
 	// Get mime type
@@ -46,7 +51,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	// Get file name
 	fileName := filepath.Base(filePath)
 	// Just use alphanumerical and some special characters for now
-	// to work around https://github.com/mattn/go-xmpp/issues/132
+	// to work around https://github.com/xmppo/go-xmpp/issues/132
 	reg := regexp.MustCompile(`[^a-zA-Z0-9\+\-\_\.]+`)
 	fileNameEscaped := reg.ReplaceAllString(fileName, "_")
 
@@ -54,16 +59,16 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	iqContent, err := sendIQ(client, iqc, jserver, "get",
 		"<query xmlns='http://jabber.org/protocol/disco#items'/>")
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 	iqDiscoItemsXML := etree.NewDocument()
 	err = iqDiscoItemsXML.ReadFromBytes(iqContent.Query)
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
-	iqDiscoItemsXMLQuery := iqDiscoItemsXML.SelectElement("query")
+	iqDiscoItemsXMLQuery = iqDiscoItemsXML.SelectElement("query")
 	if iqDiscoItemsXMLQuery == nil {
-		log.Fatal("no query element in disco items reply")
+		return "", errors.New("http-upload: no query element in disco items reply")
 	}
 	iqDiscoItemsXMLItems := iqDiscoItemsXMLQuery.SelectElements("item")
 
@@ -76,11 +81,11 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 		iqDiscoInfoReqXMLQuery.CreateAttr("xmlns", nsDiscoInfo)
 		iqdi, err := iqDiscoInfoReqXML.WriteToString()
 		if err != nil {
-			log.Fatal(err)
+			return "", err
 		}
 		iqDiscoInfo, err := sendIQ(client, iqc, jid.Value, "get", iqdi)
 		if err != nil {
-			log.Fatal(err)
+			return "", err
 		}
 		if iqDiscoInfo.Type != strResult {
 			continue
@@ -88,9 +93,9 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 		iqDiscoInfoXML := etree.NewDocument()
 		err = iqDiscoInfoXML.ReadFromBytes(iqDiscoInfo.Query)
 		if err != nil {
-			log.Fatal(err)
+			return "", err
 		}
-		iqDiscoInfoXMLQuery := iqDiscoInfoXML.SelectElement("query")
+		iqDiscoInfoXMLQuery = iqDiscoInfoXML.SelectElement("query")
 		if iqDiscoInfoXMLQuery == nil {
 			continue
 		}
@@ -109,12 +114,13 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 		if iqDiscoInfoXMLType.Value == "file" &&
 			iqDiscoInfoXMLCategory.Value == "store" {
 			uploadComponent = jid.Value
+			break
 		}
 	}
 	if uploadComponent == "" {
-		log.Fatal("No http upload component found.")
+		return "", errors.New("http-upload: no http upload component found.")
 	}
-	iqDiscoInfoXMLX := iqDiscoItemsXMLQuery.SelectElements("x")
+	iqDiscoInfoXMLX := iqDiscoInfoXMLQuery.SelectElements("x")
 	for _, r := range iqDiscoInfoXMLX {
 		field := r.SelectElements("field")
 		for i, t := range field {
@@ -122,18 +128,23 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 			if varAttr == nil {
 				continue
 			}
-			prevFieldVal := field[i-1].SelectElement("value")
-			if prevFieldVal == nil {
-				continue
-			}
 			curFieldVal := t.SelectElement("value")
 			if curFieldVal == nil {
 				continue
 			}
-			if varAttr.Value == "max-file-size" && prevFieldVal.Text() == nsHTTPUpload {
-				maxFileSize, err = strconv.ParseInt(curFieldVal.Text(), 10, 64)
-				if err != nil {
-					log.Fatal("error while checking server maximum http upload file size.")
+			if varAttr.Value == "max-file-size" {
+				var prevFieldVal *etree.Element
+				if i > 0 {
+					prevFieldVal = field[i-1].SelectElement("value")
+					if prevFieldVal == nil {
+						continue
+					}
+				}
+				if prevFieldVal.Text() == nsHTTPUpload {
+					maxFileSize, err = strconv.ParseInt(curFieldVal.Text(), 10, 64)
+					if err != nil {
+						return "", errors.New("http-upload: error while checking server maximum http upload file size.")
+					}
 				}
 			}
 		}
@@ -143,7 +154,7 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	// the best.
 	if maxFileSize != 0 {
 		if fileSize > maxFileSize {
-			log.Fatal("File size " + strconv.FormatInt(fileSize/1024/1024, 10) +
+			return "", errors.New("http-upload: file size " + strconv.FormatInt(fileSize/1024/1024, 10) +
 				" MB is larger than the maximum file size allowed (" +
 				strconv.FormatInt(maxFileSize/1024/1024, 10) + " MB).")
 		}
@@ -158,43 +169,55 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	requestReq.CreateAttr("content-type", mimeType)
 	r, err := request.WriteToString()
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 
 	// Request http upload slot
 	uploadSlot, err := sendIQ(client, iqc, uploadComponent, "get", r)
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 	if uploadSlot.Type != strResult {
-		log.Fatal("error while requesting upload slot.")
+		return "", errors.New("http-upload: error while requesting upload slot.")
 	}
 	iqHTTPUploadSlotXML := etree.NewDocument()
 	err = iqHTTPUploadSlotXML.ReadFromBytes(uploadSlot.Query)
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 	iqHTTPUploadSlotXMLSlot := iqHTTPUploadSlotXML.SelectElement("slot")
 	if iqHTTPUploadSlotXMLSlot == nil {
-		log.Fatal("http-upload: no slot element")
+		return "", errors.New("http-upload: no slot element")
 	}
 	iqHTTPUploadSlotXMLPut := iqHTTPUploadSlotXMLSlot.SelectElement("put")
 	if iqHTTPUploadSlotXMLPut == nil {
-		log.Fatal("http-upload: no put element")
+		return "", errors.New("http-upload: no put element")
 	}
 	iqHTTPUploadSlotXMLPutURL := iqHTTPUploadSlotXMLPut.SelectAttr("url")
 	if iqHTTPUploadSlotXMLPutURL == nil {
-		log.Fatal("http-upload: no url attribute")
+		return "", errors.New("http-upload: no url attribute")
 	}
 	if !strings.HasPrefix(iqHTTPUploadSlotXMLPutURL.Value, "https://") {
-		log.Fatal("http-upload: upload slot does not provide https")
+		return "", errors.New("http-upload: upload slot does not provide https")
 	}
 	// Upload file
-	httpClient := &http.Client{}
+	httpTransport := &http.Transport{
+		IdleConnTimeout:     timeout,
+		TLSHandshakeTimeout: timeout,
+	}
+	proxyEnv := os.Getenv("HTTP_PROXY")
+	if proxyEnv != "" {
+		proxyURL, err := url.Parse(proxyEnv)
+		if err != nil {
+			return "", err
+		}
+		httpTransport.Proxy = http.ProxyURL(proxyURL)
+	}
+	httpClient := &http.Client{Transport: httpTransport}
 	req, err := http.NewRequest(http.MethodPut, iqHTTPUploadSlotXMLPutURL.Value,
 		buffer)
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 	req.Header.Set("Content-Type", mimeTypeEscaped.String())
 	iqHTTPUploadSlotXMLPutHeaders := iqHTTPUploadSlotXMLPut.SelectElements("header")
@@ -210,25 +233,25 @@ func httpUpload(client *xmpp.Client, iqc chan xmpp.IQ, jserver string, filePath
 	}
 	resp, err := httpClient.Do(req)
 	if err != nil {
-		log.Fatal(err)
+		return "", err
 	}
 	// Test for http status code "200 OK" or "201 Created"
 	if resp.StatusCode != 200 && resp.StatusCode != 201 {
-		log.Fatal("Http upload failed.")
+		return "", errors.New("http-upload: upload failed.")
 	}
 
 	// Return http link
 	iqHTTPUploadSlotXMLGet := iqHTTPUploadSlotXMLSlot.SelectElement("get")
 	if iqHTTPUploadSlotXMLGet == nil {
-		log.Fatal("http-upload: no get element")
+		return "", errors.New("http-upload: no get element")
 	}
 	iqHTTPUploadSlotXMLGetURL := iqHTTPUploadSlotXMLGet.SelectAttr("url")
 	if iqHTTPUploadSlotXMLGetURL == nil {
-		log.Fatal("http-upload: no url attribute")
+		return "", errors.New("http-upload: no url attribute")
 	}
 	err = resp.Body.Close()
 	if err != nil {
-		fmt.Println("error while closing http request body:", err)
+		fmt.Println("http-upload: error while closing http request body:", err)
 	}
-	return iqHTTPUploadSlotXMLGetURL.Value
+	return iqHTTPUploadSlotXMLGetURL.Value, nil
 }
