@@ -344,7 +344,16 @@ func oxGetPrivKeyLoc(jid string) (string, error) {
 			return strError, fmt.Errorf("oxGetPrivKeyLoc: could not create folder for private keys: %w", err)
 		}
 	}
-	dataFile := dataDir + base64.StdEncoding.EncodeToString([]byte(jid))
+	// TODO: Remove handling of oldDataFile in a later version when it's very likely that there are no
+	// more versions in use using the oldDataFile (<0.8.3).
+	oldDataFile := dataDir + base64.StdEncoding.EncodeToString([]byte(jid))
+	dataFile := dataDir + strings.Replace(jid, "@", "_at_", -1)
+	if _, err := os.Stat(oldDataFile); err == nil {
+		err := os.Rename(oldDataFile, dataFile)
+		if err != nil {
+			return dataFile, err
+		}
+	}
 	return dataFile, nil
 }
 
