@@ -20,7 +20,7 @@ import (
 func findConfig() (string, error) {
 	// Get the current user.
 	curUser, err := user.Current()
-	if errors.Unwrap(err) != nil {
+	if err != nil {
 		return "", fmt.Errorf("findConfig: failed to get current user: %w", err)
 	}
 	// Get home directory.
@@ -41,7 +41,7 @@ func findConfig() (string, error) {
 	for _, r := range configFiles {
 		// Check that the config file is existing.
 		_, err := os.Stat(r)
-		if errors.Unwrap(err) == nil {
+		if err == nil {
 			return r, nil
 		}
 	}
@@ -62,7 +62,7 @@ func parseConfig(configPath string) (configuration, error) {
 	// Get systems user config path.
 	if configPath == "" {
 		configPath, err = findConfig()
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 	}
@@ -70,7 +70,7 @@ func parseConfig(configPath string) (configuration, error) {
 	// Only check file permissions if we are not running on windows.
 	if runtime.GOOS != "windows" {
 		info, err := os.Stat(configPath)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			log.Fatal(err)
 		}
 		// Check for file permissions. Must be 600, 640, 440 or 400.
@@ -84,7 +84,7 @@ func parseConfig(configPath string) (configuration, error) {
 
 	// Open config file.
 	file, err := os.Open(configPath)
-	if errors.Unwrap(err) != nil {
+	if err != nil {
 		return output, fmt.Errorf("parseConfig: failed to open config file: %w", err)
 	}
 	scanner := bufio.NewScanner(file)
@@ -111,7 +111,7 @@ func parseConfig(configPath string) (configuration, error) {
 				shell = "/bin/sh"
 			}
 			out, err := exec.Command(shell, "-c", row[1]).Output()
-			if errors.Unwrap(err) != nil {
+			if err != nil {
 				log.Fatal(err)
 			}
 			output.password = string(out)
@@ -120,9 +120,6 @@ func parseConfig(configPath string) (configuration, error) {
 			}
 		case "port:":
 			output.port = row[1]
-		case "resource:":
-			output.resource = row[1]
-			fmt.Println(configPath+":", "Deprecated option: resource.")
 		case "alias:":
 			output.alias = row[1]
 		default:
@@ -144,19 +141,19 @@ func parseConfig(configPath string) (configuration, error) {
 		}
 	}
 	err = file.Close()
-	if errors.Unwrap(err) != nil {
+	if err != nil {
 		fmt.Println("error closing file:", err)
 	}
 
 	// Check if the username is a valid JID
 	output.username, err = MarshalJID(output.username)
-	if errors.Unwrap(err) != nil {
+	if err != nil {
 		// Check whether only the local part was used by appending an @ and the
 		// server part.
 		output.username = output.username + "@" + output.jserver
 		// Check if the username is a valid JID now
 		output.username, err = MarshalJID(output.username)
-		if errors.Unwrap(err) != nil {
+		if err != nil {
 			return output, errors.New("invalid username/JID: " + output.username)
 		}
 	}
