@@ -11,7 +11,7 @@ import (
 	"runtime"
 
 	"github.com/beevik/etree"  // BSD-2-clause
-	"github.com/mattn/go-xmpp" // BSD-3-Clause
+	"github.com/xmppo/go-xmpp" // BSD-3-Clause
 )
 
 func sendIQ(client *xmpp.Client, iqc chan xmpp.IQ, target string, iQtype string, content string) (xmpp.IQ, error) {
